@@ -1,38 +1,105 @@
 """ Yet another FASTQ parsing set of tools """
 
-import io
+import warnings
+import io, gzip
 from collections import namedtuple
 
-Entry = namedtuple('Entry', 'header sequence quality')
+SeqQualEntry = namedtuple('Entry', 'header sequence quality')
+Entry = SeqQualEntry
 
 class FastqError(ValueError):
     pass
 
+
 #Compatibility with Python 3
 import sys
+_python2 = sys.version_info[0] == 2
 if sys.version_info[0] > 2:
-    import _io
-    file = _io.TextIOWrapper
+    import io
+    file = io.BufferedReader
+
+if _python2:
+    import itertools
+    zip = itertools.izip
 
 
 class FastqFile(file):
     """ FASTQ file.
-    The default iterator will go through the entries in the file (not the rows). 
+    The default iterator will go through the entries in the file (not the rows) 
     """
-        
+
+    def __init__(self, filename, **kwargs):
+        if _python2:
+            super(FastqFile, self).__init__(filename, **kwargs)
+        else:
+            stream = io.FileIO(filename)
+            super(FastqFile, self).__init__(stream, **kwargs)
+
     def __iter__(self):
         wrap = io.OpenWrapper(self.fileno(), closefd = False)
-        return read_fastq(wrap)
+        return iter_entries(wrap)
 
+    def iter_seqqual(self):
+        wrap = io.OpenWrapper(self.fileno(), closefd = False)
+        return iter_seqqual(wrap)
+        
     def readcount(self):
         pos = self.tell()
         for i, entry in enumerate(read_fastq):
             pass
         return i+1
 
-def read_fastq(stream):
+class FastqFilePair(object):
+    """ Pair of FASTQ files
+    The default iterator will go through the paired entries in the file
+    (not the rows), assuming that they are in the same order.
+    No check that this is the case (using IDs) is performed.
+    """
+
+    def __init__(self, filename1, filename2, **kwargs):
+        if _python2:
+            fh1 = FastqFile(filename1, **kwargs)
+            fh2 = FastqFile(filename2, **kwargs)
+        else:
+            stream = io.FileIO(filename1)
+            fh1 = FastqFile(stream, **kwargs)
+            stream = io.FileIO(filename2)
+            fh2 = FastqFile(stream, **kwargs)
+        self._fh1 = fh1
+        self._fh2 = fh2
+
+    def __iter__(self):
+        wrap1 = io.OpenWrapper(self._fh1.fileno(), closefd = False)
+        wrap2 = io.OpenWrapper(self._fh2.fileno(), closefd = False)
+        return iter_entry_pairs(wrap1, wrap2)
+
+    def iter_seqqual_pairs(self):
+        """ Iterate over the pairs of (sequence+quality) in 
+        the file. """
+        wrap = io.OpenWrapper(self.fileno(), closefd = False)
+        return iter_seqqual(wrap)
+        
+    def readcount(self):
+        pos = self.tell()
+        for i, entry in enumerate(read_fastq):
+            pass
+        return i+1
+    
+
+class GzipFastqFile(gzip.GzipFile):
+    """ Gzip-compressed FASTQ file. 
+    The default iterator will go over the entries in the file.
+    """
+
+    def __iter__(self):
+        return read_fastq(self)
+
+    def iter_seqqual(self):
+        return iter_seqqual(self)
+
+def iter_seqqual(stream):
     """ Return an iterator over FASTQ entries in a stream
-    (an iterable) """
+    (an iterable) of _bytes_ """
 
     sequence = list()
     quality = list()
@@ -85,8 +152,22 @@ def read_fastq(stream):
     if header is None:
         raise FastqError('Missing FASTQ header.')
     if row != header:
-        yield Entry(header, sequence, quality)        
+        yield SeqQualEntry(header, sequence, quality)        
 
+def iter_seqqual_pairs(stream1, stream2):
+    return zip(stream1, stream2)
+
+def iter_entries(stream):
+    """ Iterate through entries in a FASTQ file. """
+    return iter_seqqual(stream)
+
+def iter_entry_pairs(stream):
+    """ Iterate through entries in a FASTQ file. """
+    return iter_seqqual_pairs(stream)
+
+def read_fastq(stream):
+    warnings.warn('read_fastq() is deprecated; use iter_entries()', DeprecationWarning)
+    return iter_entries(stream)
 
 def make_htmlreport(fqfile, directory = '.', verbose = True,
                     sample_percent = 0.05):
